/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.chart.sw;

import net.moioli.chart.GraphicsI;
import waba.fx.Color;
import waba.fx.FontMetrics;
import waba.fx.Graphics;
import waba.ui.MainWindow;
import waba.util.Random;

/**
 * @author silvio
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class SuperWabaGraphicsWrapper implements GraphicsI {
        private Graphics g = null;
        private static Random rg = new Random();
        private FontMetrics fm = MainWindow.getMainWindow().createGraphics().getFont().fm;
        
        //non chiamabile dall'esterno
        private SuperWabaGraphicsWrapper(){
            //Never get here
        }
        
        /**
         * Costruttore di default
         */
        public SuperWabaGraphicsWrapper(Graphics g){
            this.g=g;
        }
        
        /**
         * Cambia il colore con cui vengono disegnati gli oggetti attraverso questa
         * interfaccia. L'oggetto passato deve essere compatibile con le classi
         * grafiche sottostanti.
         * 
         * @param c il nuovo colore
         */
        public void setColor(Object c){
            this.g.setForeColor((waba.fx.Color) c);
            this.g.setBackColor((waba.fx.Color) c);
        }

        /**
         * Disegna una linea retta dal punto (x1,y1) al punto (x2,y2).
         * 
         * @param x1 ascissa del primo punto
         * @param y1 ordinata del primo punto
         * @param x2 ascissa del secondo punto
         * @param y2 ordinata del secondo punto
         */
        public void drawLine(int x1, int y1, int x2, int y2){
            this.g.drawLine(x1,y1,x2,y2);
        }

        /**
         * Disegna una stringa a partire dal punto (x,y).
         * 
         * @param s la stringa da disegnare
         * @param x ascissa del punto
         * @param y ordinata del punto
         */
        public void drawString(String s, int x, int y){
            this.g.drawText(s,x,y);
        }

        /**
         * Disegna un poligono pieno con i vertici passati.
         * 
         * @param xPoints array delle ascisse dei vertici
         * @param yPoints array delle ordinate dei vertici
         * @param n numero dei vertici
         */
        public void fillPolygon(int[] xPoints, int[] yPoints, int n){
            this.g.fillPolygon(xPoints,yPoints,n);
        }

        /**
         * Disegna un cerchio di coordinate e raggio specificati.
         * 
         * @param x ascissa del centro
         * @param y ordinata del centro
         * @param r raggio del cerchio
         */
        public void drawCircle(int x, int y, int r){
            int rm = r/2;
            this.g.drawArc(x+rm,y+rm,rm,0,359);
        }

        /**
         * Ritorna la larghezza della stringa una volta disegnata in pixel.
         * 
         * @param s la stringa in esame
         * @return il valore
         */
        public int getStringWidth(String s){
            return this.fm.getTextWidth(s);
        }

        /**
         * Ritorna l'altezza massima di una stringa una volta disegnata in pixel.
         * @return il valore
         */
        public int getStringHeight(){
            return this.fm.height;
        }
        /**
         * Ritorna la rappresentazione del colore nero per le classi grafiche
         * sottostanti.
         * 
         * @return il nero
         */
        public Object getBlackColor(){
            return waba.fx.Color.BLACK;
        }

        /**
         * Ritorna la rappresentazione di un colore per le classi grafiche
         * sottostanti. A n diversi devono (possibilmente) corrispondere colori
         * diversi.
         * 
         * @param n l'indice del colore
         * @return un colore
         */
        public Object getRandomColor(){
            //For graphs use, better dark colors.
            return new Color(rg.nextInt(0x00FFFFFF)).darker(5);
        }
    }