package sxwaba.ui;

import waba.ui.*;
import waba.fx.*;
import waba.sys.Vm;
import waba.util.Vector;
import waba.util.IntVector;
/**
* \author ian reinhart geiser  <geiseri@sourcextreme.com>
* \version "$Id: HBox.java,v 1.3 2004/03/31 02:36:03 geiseri Exp $"
* \date 2003
* \class HBox
* \brief A horizontal box implementation of the box layout.
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* An example of use is as follows:
* \code
* hbox = new HBox();
* label = new Label("Test");
* button = new Button("Test");
* edit = new Edit("");
* edit.setMode(Edit.CURRENCY);
* edit.setValidChars("1234567890.");
* edit.setKeyboard(Edit.KBD_CALCULATOR);
* hox.add(label,0);		// Default horizontal size
* hbox.add(edit,2);		// Stretch 2x all other horizontal controls
* hbox.add(button,4);		// Stretch 4x all other horizontal controls
* hbox.setRect(getClientRect());
* \endcode
*/
public class HBox extends BoxBase
{

	/**
	* Creates a new HBox Layout implementation.
	*/
	public HBox()
	{
		super();
	}
	public int getPreferredWidth()
	{
		int returnWidth = m_hSpc;
		int size = m_controls.size();
		for ( int idx = 0; idx < size; ++idx )
		{
			Control ctl = ( Control ) m_controls.items[idx];
			returnWidth += ctl.getPreferredWidth()+ m_hSpc;
		}

		/*
		  waba.sys.Vm.debug("My preferredWidtht " + String.valueOf(super.getPreferredWidth()));
		  waba.sys.Vm.debug("Get preferredWidth " + String.valueOf(returnWidth));
		*/
		return returnWidth + m_spc;
	}
	
	public int getPreferredHeight()
	{
		int returnHeight = 0;
		int size = m_controls.size();
		for ( int idx = 0; idx < size; ++idx )
		{
			Control ctl = ( Control ) m_controls.items[idx];
			returnHeight = Math.max( ctl.getPreferredHeight(), returnHeight);
		}
		/*
			waba.sys.Vm.debug("My preferredHeighth " + String.valueOf(super.preferredHeight()));
			waba.sys.Vm.debug("Get preferredHeight " + String.valueOf(returnHeight));
		*/
		return returnHeight + m_pad;
	}
	
	/**
	* Recalculate the unmanaged width of the box.  This allows the preferred size of Controls to change and be
	* reflected later when the layout changes.
	*/
	private int unmanagedWidth()
	{
		int m_unmanagedWidth = 0;
		int size = m_controls.size();
		Control c;

		for( int idx = 0; idx < size; ++idx )
		{
			if( m_stretch.items[idx] == 0 )
			{
				c = ( Control ) m_controls.items[idx];
				/*
					waba.sys.Vm.debug("Preferred Width for " + c.toString() );
					waba.sys.Vm.debug("W: " +  String.valueOf(c.getPreferredWidth()));
				*/
				m_unmanagedWidth += c.getPreferredWidth();
			}
		}

		return m_unmanagedWidth;
	}

	/**
	* Reimplementation of Control.onBoundsChanged().
	* Recursively applies the stretch to a layout after setRect() is called.
	*/
	protected void onBoundsChanged()
	{
		int X = 0;
		int Y = 0;
		int W = width - ( unmanagedWidth() + (m_controls.size()*m_spc));
		int prevX = X;
		int size = m_controls.size();

		//waba.sys.Vm.debug("Laying out " + this.toString() );
		if ( W < 0 )
		{
			W = 0;  // Yeah your layout is hozed at this point anyway...
			waba.sys.Vm.debug("Warning Layout Hozed!");
		}

		/*
		  waba.sys.Vm.debug("HBox...");
		  waba.sys.Vm.debug("Width: "+ String.valueOf(width) +" Unmanaged Width: " + String.valueOf( unmanagedWidth()));
		*/

		for ( int idx = 0; idx < size; ++idx )
		{
			Control ctl = ( Control ) m_controls.items[idx];
			//waba.sys.Vm.debug("Laying out control " + ctl.toString() );
			int newWidth = 0;
			if ( m_stretch.items[idx] != 0)
			{
				newWidth =(( m_stretch.items[idx] * W)) / m_total;
				//waba.sys.Vm.debug("HBox - calculated " + String.valueOf(newWidth));
			}
			else
			{
				newWidth = ctl.getPreferredWidth() + m_pad;
				//waba.sys.Vm.debug("HBox - preferred" + String.valueOf(newWidth));
			}

			if ( ( newWidth + prevX +  m_hSpc ) > width)
			{
				newWidth = width - (prevX +  m_hSpc);
				//waba.sys.Vm.debug("Clipping to " + String.valueOf(newWidth));
			}
			if ( newWidth < 1)
				newWidth = 1;
			//waba.sys.Vm.debug("X: " + String.valueOf(prevX +  m_hSpc) + " Y: " + String.valueOf(Y +  m_hSpc) + " W: " + String.valueOf(newWidth) + " H: "+ String.valueOf(height -  m_hSpc));
			ctl.setRect( prevX+  m_hSpc +   m_hSpc, Y+  m_hSpc , newWidth, height -   m_hSpc );
			prevX += newWidth +   m_hSpc;
		}
	}
}
