package sxwaba.ui;

import waba.ui.*;
import waba.fx.*;
import waba.sys.Vm;
import waba.util.Vector;
import waba.util.IntVector;
/**
* \author ian reinhart geiser  <geiseri@sourcextreme.com>
* \version "$Id: VBox.java,v 1.3 2004/03/31 02:36:04 geiseri Exp $"
* \date 2003
* \class VBox
* \brief A vertical box implementation of the box layout.
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* An example of use is as follows:
* \code
* vbox = new VBox();
* label = new Label("Test");
* button = new Button("Test");
* edit = new Edit("");
* edit.setMode(Edit.CURRENCY);
* edit.setValidChars("1234567890.");
* edit.setKeyboard(Edit.KBD_CALCULATOR);
* vox.add(label,0);		// Default vertical size
* vbox.add(edit,2);		// Stretch 2x all other vertical controls
* vbox.add(button,4);		// Stretch 4x all other vertical controls
* vbox.setRect(getClientRect());
* \endcode
*/
public class VBox extends BoxBase
{
	/**
	* Creates a new VBox layout implementation.
	*/
	public VBox()
	{
		super();
	}

	public int getPreferredHeight()
	{
		int returnHeight = m_spc;
		int size = m_controls.size();
		for ( int idx = 0; idx < size; ++idx )
		{
			Control ctl = ( Control ) m_controls.items[idx];
			returnHeight += ctl.getPreferredHeight()+ m_hSpc;
		}

		/*
			waba.sys.Vm.debug("My preferredHeight " + String.valueOf(super.getPreferredHeight()));
			waba.sys.Vm.debug("Get preferredHeight " + String.valueOf(returnHeight));
		*/
		return returnHeight + m_spc;
	}
	
	public int getPreferredWidth()
	{
		int returnWidth = 0;
		int size = m_controls.size();
		for ( int idx = 0; idx < size; ++idx )
		{
			Control ctl = ( Control ) m_controls.items[idx];
			returnWidth = Math.max( ctl.getPreferredWidth(), returnWidth);
		}
		/*
			waba.sys.Vm.debug("My preferredWidth " + String.valueOf(super.preferredWidth()));
			waba.sys.Vm.debug("Get preferredWidth " + String.valueOf(returnWidth));
		*/
		return returnWidth + m_pad;
	}

	/**
	* Recalculate the unmanaged height of the box.  This allows the preferred size of Controls to change and be
	* reflected later when the layout changes.
	*/
	private int unmanagedHeight()
	{
		int m_unmanagedHeight = 0;
		int size = m_controls.size();
		Control c;

		for( int idx = 0; idx < size; ++idx )
		{
			if( m_stretch.items[ idx ] == 0 )
			{
				c = ( Control ) m_controls.items[idx];
				/*
					waba.sys.Vm.debug("Preferred Height for " + c.toString() );
					waba.sys.Vm.debug("H: " +  String.valueOf(c.getPreferredHeight()));
				*/
				m_unmanagedHeight += c.getPreferredHeight();
			}
		}

		return m_unmanagedHeight;
	}

	/**
	* Reimplementation of Control.onBoundsChanged().
	* Recursively applies the stretch to a layout after setRect() is called.
	*/
	protected void onBoundsChanged()
	{
		int X = 0;
		int Y = 0;
		int H = height - ( unmanagedHeight() + (m_controls.size()*m_spc));

		int prevY = Y;
		int size = m_controls.size();

		if ( H < 0 )
		{
			H = 0;  // Yeah your layout is hozed at this point anyway...
			//waba.sys.Vm.debug("Warning Layout Hozed!");
		}

		/*
			waba.sys.Vm.debug("VBox...");
			waba.sys.Vm.debug("Height: "+ String.valueOf(height) +" Unmanaged Height: " + String.valueOf( unmanagedHeight()));
		*/

		for ( int idx = 0; idx < size; ++idx )
		{
			Control ctl = ( Control ) m_controls.items[idx];
			int newHeight =0;
			if ( m_stretch.items[idx] != 0)
			{
				newHeight = ((m_stretch.items[idx] * H)) / m_total;
				//waba.sys.Vm.debug("VBox - calculated " + String.valueOf(newHeight) );
			}
			else
			{
				newHeight = ctl.getPreferredHeight() + m_pad;
				//waba.sys.Vm.debug("VBox - preferred " + String.valueOf(newHeight));
			}
			if ( ( newHeight + prevY + m_spc ) > height)
			{
				newHeight = height - (prevY + m_spc);
				//waba.sys.Vm.debug("Clipping " + String.valueOf(newHeight));

			}

			if ( newHeight < 1)
				newHeight = 1;
			//waba.sys.Vm.debug("X: " + String.valueOf(X+   m_hSpc) + " Y: " + String.valueOf(prevY+  m_hSpc) + " W: " + String.valueOf(width-  m_hSpc) + " H: "+ String.valueOf(newHeight));
			ctl.setRect( X +   m_hSpc , prevY+   m_hSpc, width -   m_hSpc, newHeight );
			prevY += newHeight +   m_hSpc;
		}
	}
}
